/* global FicheQL */

FicheQL.Fiche = function (options) {
    this._corpusArray = new Array();
    this._rangeArray = new Array();
    this._redacteurArray = new Array();
    this._sphereArray = new Array();
    this._motcleQueryArray = new Array();
    this.withCurrentCorpus  = false;
    this.discardFilter = FicheQL.DISCARDFILTER_ALL;
    this.withGeoloc  = false;
    this.periodCondition = null;
    this.fieldQuery = null;
    this.redacteurFilter = FicheQL.REDACTEURFILTER_ALL;
    this.motcleSelectMode = FicheQL.MODE_AND;
    this.withRattachement = false;
    this.croisementCondition = null;
    if (options) {
        for(let key in options) {
            if (key.indexOf('_') !== 0) {
                var value = options[key];
                switch(key)  {
                    case "corpus":
                        this.addCorpus(value);
                        break;
                    case "croisement":
                        this.setCroisementCondition(value);
                        break;
                    case "current":
                        this.withCurrentCorpus = value;
                        break;
                    case "field-query":
                    case "fieldQuery":
                    case "field-select":
                    case "fieldSelect":
                        this.setFieldQuery(value);
                        break;
                    case "geoloc":
                        this.withGeoloc = value;
                        break;
                    case "motcle-query":
                    case "motcleQuery":
                    case "motcle-select":
                    case "motcleSelect":
                        this.addMotcleQuery(value);
                        break;
                    case "period":
                        this.periodCondition = new FicheQL.Condition.Period(value);
                        break;
                    case "redacteurs":
                        this.addRedacteurs(value);
                        break;
                    case "range":
                        this._rangeArray = FicheQL.toRangeArray(value);
                        break;
                    case "rattachement":
                        this.withRattachement = value;
                        break;
                    case "spheres":
                        this.addSpheres(value);
                        break;
                    default:
                        this[key] = value;
                }
            }
        }
    }
};

FicheQL.Fiche.prototype.addCorpus = function (corpusNames) {
    var tmparray;
    if (Array.isArray(corpusNames)) {
        tmparray = corpusNames;
    } else {
        tmparray = corpusNames.split(" ");
    }
    for(let crpVal of tmparray) {
        if (crpVal.length > 0) {
            this._corpusArray.push(crpVal);
        }
    }
    return this;
};

FicheQL.Fiche.prototype.addSpheres = function (sphereNames) {
    var tmparray;
    if (Array.isArray(sphereNames)) {
        tmparray = sphereNames;
    } else {
        tmparray = sphereNames.split(" ");
    }
    for(let sphVal of tmparray) {
        if (sphVal.length > 0) {
            this._sphereArray.push(sphVal);
        }
    }
    if (this._sphereArray.length > 0) {
        this.redacteurFilter = FicheQL.REDACTEURFILTER_SOME;
    }
    return this;
};

FicheQL.Fiche.prototype.addRedacteurs = function (redacteurs) {
    var tmparray;
    if (Array.isArray(redacteurs)) {
        tmparray = redacteurs;
    } else {
        tmparray = redacteurs.split(" ");
    }
    for(let redacteur of tmparray) {
        if (redacteur.length > 0) {
            this._redacteurArray.push(redacteur);
        }
    }
    if (this._redacteurArray.length > 0) {
        this.redacteurFilter = FicheQL.REDACTEURFILTER_SOME;
    }
    return this;
};

FicheQL.Fiche.prototype.addRange = function (start, end) {
    this._rangeArray = FicheQL.addRange(this._rangeArray, start, end);
    return this;
};

FicheQL.Fiche.prototype.addMotcleQuery = function (motcleQuery) {
    if (Array.isArray(motcleQuery)) {
        for(let val of motcleQuery) {
            if (val) {
                if (val instanceof FicheQL.Motcle) {
                    this._motcleQueryArray.push(val);
                } else {
                    this._motcleQueryArray.push(new FicheQL.Motcle(val));
                }
            }
        }
    } else if (motcleQuery instanceof FicheQL.Motcle) {
        this._motcleQueryArray.push(motcleQuery);
    } else {
        this._motcleQueryArray.push(new FicheQL.Motcle(motcleQuery));
    }
    return this;
};

FicheQL.Fiche.prototype.setFieldQuery = function (obj) {
    if (obj instanceof FicheQL.Field) {
        this.fieldQuery = obj;
    } else {
        this.fieldQuery = new FicheQL.Field(obj);
    }
};

FicheQL.Fiche.prototype.setCroisementCondition = function (obj) {
    if (obj instanceof FicheQL.Condition.Croisement) {
        this.croisementCondition = obj;
    } else {
        this.croisementCondition = new FicheQL.Condition.Croisement(obj);
    }
};

FicheQL.Fiche.prototype.toXml = function (indent) {
    if ((!indent) && (indent !== 0)) {
        indent = -999;
    }
    var xml = '';
    _indent();
    xml += '<fiche-query>';
    indent++;
    for(let corpus of this._corpusArray) {
        _indent();
        xml += '<corpus>' + FicheQL.escape(corpus) + '</corpus>';
    }
    if (this.withCurrentCorpus) {
        _indent();
        xml += '<current/>';
    }
    var rangeLength = this._rangeArray.length;
    if (rangeLength > 0) {
        _indent();
        xml += '<range>';
        for(let i = 0; i < rangeLength; i++) {
            if (i > 0) {
                xml += ';';
            }
            xml += FicheQL.escape(this._rangeArray[i]);
        }
        xml += '</range>';
    }
    if (this.discardFilter !== FicheQL.DISCARDFILTER_ALL) {
        _indent();
        xml += '<discard filter="' + FicheQL.escape(this.discardFilter) + '"/>';
    }
    if ((this.periodCondition) && (this.periodCondition.toXml)) {
        xml += this.periodCondition.toXml(indent);
    }
    if ((this.fieldQuery) && (this.fieldQuery.toXml)) {
        xml += this.fieldQuery.toXml(indent);
    }
    if (this.redacteurFilter === FicheQL.REDACTEURFILTER_SOME) {
        let redacteurLength = this._redacteurArray.length;
        let sphereLength = this._sphereArray.length;
        if ((redacteurLength > 0) || (sphereLength.length > 0)) {
            _indent();
            xml += '<redacteurs>';
            indent++;
            for(let i = 0; i < redacteurLength; i++) {
                _indent();
                xml += '<redacteur>' +  FicheQL.escape(this._redacteurArray[i]) + '</redacteur>';
            }
            for(let i = 0; i < sphereLength; i++) {
                _indent();
                xml += '<sphere>' +  FicheQL.escape(this._sphereArray[i]) + '</sphere>';
            }
            indent--;
            _indent();
            xml += '</redacteurs>';
        }
    } else if (this.redacteurFilter !== FicheQL.REDACTEURFILTER_ALL) {
         _indent();
        xml += '<redacteurs filter="' +  FicheQL.escape(this.redacteurFilter) + '"/>';
    }
    if (this.withGeoloc) {
         _indent();
         xml += '<geoloc/>';
    }
    if (this._motcleQueryArray.length > 1) {
        _indent();
        xml += '<motcle-query-mode mode="' + FicheQL.escape(this.motcleSelectMode) + '"/>';
    }
    for(let motcleQuery of this._motcleQueryArray) {
        if ((motcleQuery) && (motcleQuery.toXml)) {
            xml += motcleQuery.toXml(indent);
        }
    }
    if (this.withRattachement) {
         _indent();
         xml += '<rattachement/>';
    }
    if ((this.croisementCondition) && (this.croisementCondition.toXml)) {
        xml += this.croisementCondition.toXml(indent);
    }
    indent--;
    _indent();
    xml += '</fiche-query>';
    return xml;
    
    
    function _indent() {
        if (indent >= 0) {
            xml += '\n';
            for(let i = 0; i < indent; i++) {
                xml += '\t';
            }
        }
    }
    
};
