/* global BDF,ID,$$,FicheQL */
/**
 * Objet global définissant l'espace de nom Menu
 * 
 * @namespace Menu
 */
var Menu = {};

Menu.ARGS = {
    bdfUser: {
      sphere: "",
      login: "",
      name: "",
      lang: "",
      locale: "",
      admin: false
    },
    links: {
        administration: false
    },
    actionGroupArray: []
};
Menu.corpusTree = [];

Menu.init = function (args) {
    var bdfUser = args.bdfUser;
    var corpusMap = new Map();
    var ficheGotoElementArray = new Array();
     $$("main_menu").html(BDF.render("menu:menu", {
        bdfUser: bdfUser,
        links: args.links,
        actionGroupArray: args.actionGroupArray
    }));
    _initGoto();
    $$("menu_action_select").change(_doAction);
    $$("menu_fiche_create").click(function () {
        this.href = "edition?corpus=" + _getSelectedCorpusName() + "&page=fiche-creation";
        return true;
    });
    BDF.registerFunction("menu:updateCorpusTree", _updateCorpusTree);
    BDF.Ajax.loadSubsetTrees("corpus", false, function (subsetTrees) {
        _updateCorpusTree(subsetTrees.corpus);
    });
       

    function _doAction() {
        let option = this.options[this.selectedIndex];
        let $option = $(option);
        let target, url;
        if (!__checkValue()) {
            return;
        }
        $$("menu_action_redo").html(BDF.render("menu:actionbutton", {
            url: url,
            text: option.text,
            target: target,
            action: $option.data("action"),
            iconSrc: $option.data("iconSrc")
        }));
        parent[target].document.location = url;
        this.selectedIndex = 0;


        function __checkValue() {
            let value = option.value;
            if (value.length === 0) {
                return false;
            }
            url = value;
            target = $option.data("target");
            if (!parent[target]) {
                alert("Unknown target : " + target);
                return false;
            }
            return true;
        }
    }
    
    function _goTo(link) {
        let currentHref = link.getAttribute("href");
        if (currentHref.length > 1) {
            return true;
        }
        let corpus = corpusMap.get(_getSelectedCorpusName());
        if (!corpus) {
            return false;
        }
        let gotoValue = ID("menu_fiche_goto").value.trim();
        if (!gotoValue) {
            _alertGoto();
            return false;
        }
        let gotoId = parseInt(gotoValue);
        if (isNaN(gotoId)) {
            if (corpus._isIdalphaSatellite) {
                let motcleQL = new FicheQL.Motcle({
                   q: gotoValue,
                   contentScope: FicheQL.SCOPE_IDALPHA_ONLY,
                   thesaurus: corpus.master.name
                });
                BDF.Ajax.loadMotcleArray(motcleQL.toXml(), function (array) {
                    if (array.length === 1) {
                        _updateGoto(corpus.name, array[0].id);
                        link.click();
                    } else {
                        alert(BDF.Loc.get("_ error.unknown.id", gotoValue));
                    }
                });
                return false;
            } else {
                _alertGoto();
            }
            return false;
        } else {
            if (gotoId < 1) {
                _alertGoto();
                return false;
            } else {
                _updateGoto(corpus.name, gotoId);
                return true;
            }
        }
    }
    
    function _alertGoto() {
        alert(BDF.Loc.get("_ error.wrong.goto"));
    }
    
    function _updateCorpusTree(corpusTree) {
        Menu.corpusTree = corpusTree;
        corpusMap.clear();
        let currentCorpus = _getSelectedCorpusName();
        __complete(corpusTree, "");
        $$("menu_fiche_select").html(BDF.render("menu:corpustree", {
            corpusTree: corpusTree
        }));
        if (corpusTree.length > 0) {
            $$("menu_fiche").removeClass("invisible");
        } else {
            $$("menu_fiche").addClass("invisible");
        }
        
        
        function __complete(array, indent) {
            for(let node of array) {
                node.indent = indent;
                if (node.node === "group") {
                    __complete(node.array, indent + "\u00A0 \u00A0 ");
                } else {
                    if (node.name === currentCorpus) {
                        node._selected = true;
                    }
                    corpusMap.set(node.name, node);
                    if ((node.master) && (node.master.type) && (node.master.type === "idalpha")) {
                        node._isIdalphaSatellite = true;
                    }
                }
            }
        }
    }
    
    function _getSelectedCorpusName() {
        let select = ID("menu_fiche_select");
        if (select.selectedIndex < 0) {
            return "";
        }
        return select.options[select.selectedIndex].value;
    }
    
    function _initGoto() {
        let $gotoElements = BDF.$({ficheGotoKey: true});
         $gotoElements.click(function () {
            return _goTo(this);
        });
         $gotoElements.each(function (index, element) {
             ficheGotoElementArray.push(element);
         });
        $$("menu_fiche_goto").on("input", function () {
            for(let element of ficheGotoElementArray) {
                element.href = "#";
                element.setAttribute("href", "#");
            }
        });
    }
    
    function _updateGoto(corpusName, ficheId) {
        for(let element of ficheGotoElementArray) {
            element.href = Menu.toHref(element.dataset.ficheGotoKey, corpusName, ficheId);
        }
    }
    
};

Menu.toHref = function (key, corpusName, ficheId) {
    switch(key) {
        case "read":
            return "fiches/" + corpusName + "-" + ficheId + ".html";
        case "write":
            return "edition?corpus=" + corpusName + "&id=" + ficheId + "&page=fiche-change";
        case "indexation":
            return "edition?corpus=" + corpusName + "&id=" + ficheId + "&page=fiche-indexation";
        case "selection":
            return "selection?corpus=" + corpusName + "&id=" + ficheId + "&page=main:fiches&cmd=SelectionByFiche";
        default:
            return "#";
    }
};


/******************************************************************************
 * Initialisation
 ******************************************************************************/

$(function () {
    BDF.initTemplates();
    Menu.init(Menu.ARGS);
});